@@ -75,7 +75,7 @@ sub connect
     $server || ($server = $main'nntpServer || $ENV{"NNTPSERVER"}) ||
 	(open(servfile) ? chop($server = <servfile>) : "");
 
-    die $noservermsg if ! $server;
+    die $noservermsg unless $server;
      ($conn = &chat'open_port($server, 119)) ||
 	die sprintf($badservermsg, $server, $!);
 
@@ -84,7 +84,7 @@ sub connect
 		 "^5.. .*$cr", 'die "NNTP-server error: " . $&',
 		 "TIMEOUT", 'die "NNTP-server connection timeout: $server\n"');
 #    print "msg= <$msg>\n";
-    $conn;
+    ($conn,$server);
 }
 
 #***********************************************************************
@@ -126,9 +126,9 @@ sub getgroups
     
 #    print "getgroups\n";
 
-    if (! @groups)
+    unless (@groups)
     {
-	&chat'print($conn, "list\n");
+	&chat'print($conn, "list\r\n");
 	    &chat'expect($conn, 100,
 			 "^215 .*$cr", ';',
 			 "^5.. .*$cr", '$done = 1, warn "$&"');
@@ -140,6 +140,7 @@ sub getgroups
 	    $buffer = &chat'expect($conn, 100,
 			 '^[\000-\377]+$', '$&',
 			 TIMEOUT, '$done = 1');
+#	print "BUFFER = <$buffer>\n";
             $buffer =~ tr/\r//d;
             $done = $done || ($buffer =~ /^\.\n/);
             $totalBuffer .= $buffer;
@@ -154,7 +155,7 @@ sub getgroups
 	{
             split;
 	    $groups{$_[0]} = ($_[1]+0)." ".($_[2]+0)." $_[3]" if $_[0] !~ /:/;
-#print "<$_[0]> = <$groups{$_[0]}>\n";
+# print "<$_[0]> = <$groups{$_[0]}>\n";
 
 #            ($ng = $_) =~ s/ .*//;
 #            $groups{$ng} = $_;
@@ -176,7 +177,7 @@ sub list
     local(@grouplist);
 
     # get an array of all the groups known at the server
-    &getgroups($conn) if ! @groups;
+    &getgroups($conn) unless @groups;
 
 # print join("\n",@groups),"\n";
     @grouplist = @groups;	# copy list of groups
@@ -191,11 +192,11 @@ sub setgroup
     local($ng) = $_[1];
     local($artcls);
 
-#    &getgroups($conn) if ! @groups;
+#    &getgroups($conn) unless @groups;
 
 #    defined($groups{$ng}) || return ();
 #  print "goup $ng\n";
-    &chat'print($conn, "group $ng\n");
+    &chat'print($conn, "group $ng\r\n");
     ($artcls = &chat'expect($conn, 100,
 			    "^211 .*$cr", '$&',
 			    "^[45].. .*$cr", '()')) || return ();
@@ -212,7 +213,7 @@ sub msgtext
     local($fail);
     local(@msg,$string,$buffer);
 
-    &chat'print($conn, "$cmd $num\n");
+    &chat'print($conn, "$cmd $num\r\n");
     $msgok = &chat'expect($conn, 100,
 			  "^220 .*$cr", '1',
 			  "^221 .*$cr", '1',
@@ -294,7 +295,7 @@ sub field
 
     @flds = ();
 
-    &chat'print($conn, "xhdr $flname $num\n");
+    &chat'print($conn, "xhdr $flname $num\r\n");
     &chat'expect($conn, 100,
 		 "^221 .*$cr", '1',
 		 "^4.. .*$cr", '0',
@@ -322,7 +323,7 @@ sub fields
 
     @flds = ();
 
-    &chat'print($conn, "xhdr $flname $first-$last\n");
+    &chat'print($conn, "xhdr $flname $first-$last\r\n");
     &chat'expect($conn, 100,
 		 "^221 .*$cr", '1',
 		 "^4.. .*$cr", '0',
@@ -446,7 +447,7 @@ sub getrc
     local ($fname) = $_[0];
     local ($ng, $artcls);
 
-    if (! open(RC, "<$fname"))
+    unless (open(RC, "<$fname"))
     {
 	warn("Can't open $fname: $!");
         foreach (keys %groups) {
@@ -454,9 +455,9 @@ sub getrc
 	    push(@ngorder, $_);
 	    $subscribed{$ng} = 0;
         }
-        $subscribed{"news.announce.important"}=1;
-        $subscribed{"news.announce.newusers"}=1;
-        $subscribed{"news.newusers.questions"}=1;
+        $subscribed{"news.announce.important"} = 1;
+        $subscribed{"news.announce.newusers"}  = 1;
+        $subscribed{"news.newusers.questions"} = 1;
         $newsrc{0} = 1;	# don't do this very often
 	return;
     }
@@ -465,6 +466,16 @@ sub getrc
     {
 	chop;
 	($ng, $artcls) = split(/[!: \t]+/, $_, 2);
+
+        unless ($groups{$ng}) {
+	    print "ignoring unknown group $ng in $fname\n";
+	    next;
+	}
+        if ($newsrc{$ng}) {
+	    print "ignoring duplicate entry for $ng in $fname\n";
+	    next;
+	}
+
 	$newsrc{$ng} = ($artcls =~ /^\d.*\d$/) ? $artcls : "0-0";
 	$subscribed{$ng} = /^[a-zA-Z0-9.+-]+:/;
 	push(@ngorder, $ng);
@@ -482,7 +493,7 @@ sub newsrc_get
 {
     local ($fname, $ng, $force) = @_;
 
-    &getrc($fname) if ! $newsrc{0};
+    &getrc($fname) unless $newsrc{0};
     $force ? $newsrc{$ng}
         : $subscribed{$ng} && $newsrc{$ng};
 }
@@ -495,43 +506,65 @@ sub subscribed {
     local (@result,$ng,$unread,$_);
 
     local($starttime) = time;
+    local($gstarttime) = time;
+    &getgroups($conn) unless @groups;
+    local($gendtime) = time;
+    local($nstarttime) = time;
+    &getrc($fname) unless $newsrc{0};
+    local($nendtime) = time;
 
-    &getgroups($conn) if !@groups;
-    &getrc($fname) if ! $newsrc{0};
-
-    undef @newNewsGroup;
-       foreach (keys %groups) {
-            $newNewsGroup{$_} = 1 if !$newsrc{$_};
-        }
-
-    foreach $ng (grep($newNewsGroup{$_},keys %newNewsGroup),@ngorder) {
-       next if !$subscribed{$ng} && $newsrc{$ng} && ($mode ne "all available");
-
-       local($unread,$_,$line) = &groupLine($ng);
-       push(@result, $line)
-              if  $unread || ($mode eq "all subscribed") || ($mode eq "all available");
+    local($nnstarttime) = time;
+    unless (defined(%newNewsGroup)) {
+	foreach (keys %groups) {
+	    $newNewsGroup{$_} = 1 unless $newsrc{$_};
+	}
+	$newNewsGroup{"dummy"}=0;
     }
-    print "time in subscribed = ",(time-$starttime)," Sekunden\n";
+    local($nnendtime) = time;
+
+    local($rstarttime) = time;
+    foreach $ng (grep($newNewsGroup{$_},keys %newNewsGroup),@ngorder) {
+	next if 
+	    !$subscribed{$ng}
+	    && $newsrc{$ng}
+	    && ($mode ne "all available");
+
+	local($unread,$range,$line);
+	($_ = $groupLine{$ng}) && (($unread,$range,$line) = split($;))
+	    || (($unread,$range,$line) = &groupLine($ng));
+#	print "unread <$unread> line <$line>\n";
+	push(@result, $line) if $unread || $mode ne "with new articles";
+    }
+    print "time in subscribed = ",(time-$starttime)," Sekunden, ",
+              ($gendtime-$gstarttime)," + ",
+              ($nendtime-$nstarttime)," + ",
+              ($nnendtime-$nnstarttime)," + ",
+              (time-$rstarttime),
+		  "\n";
     return @result;
 }
 
 sub groupLine {
     local($ng) = @_;
-    local($unread,$rangelist,$line) = split($;,$groupLine{$ng});
-    return ($unread,$rangelist,$line) if $line;
+#    local($unread,$rangelist,$line) = split($;,$groupLine{$ng});
+#    return ($unread,$rangelist,$line) if $line;
 
+    local($subscribed,$entry) = ($subscribed{$ng},$newsrc{$ng});     
     local($last,$first) = (split(? ?,$groups{$ng}))[(0,1)];
     local($articles) = &canon_count("$first-$last");
-    local($entry) = $newsrc{$ng};
-    $rangelist =  $entry ? 
-#	&canon_inverse(&canon_artlist($entry),$first,$last) : "$first-$last";
-	&canon_inverse($entry,$first,$last) : "$first-$last";
-    $unread = &canon_count($rangelist);
 
+    unless ($subscribed) {
+	$rangelist="$first-$last";
+        $unread=$articles;
+    } else {
+	$rangelist =  $entry ? 
+	    &canon_inverse($entry,$first,$last) : "$first-$last";
+	$unread = &canon_count($rangelist);
+    }
 # print "counting $ng <$rangelist> returns $unread in range <$first-$last>\n";
 
     $line = sprintf("%-45s %4d unread + %4d read ", $ng , $unread,$articles-$unread)
-		 . ($entry ? ($subscribed{$ng} ? "" : "(unsubscribed)") : "(new)");
+		 . ($entry ? ($subscribed ? "" : "(unsubscribed)") : "(new)");
 
     $groupLine{$ng} = "$unread$;$rangelist$;$line";
     return ($unread,$rangelist,$line);
@@ -543,8 +576,8 @@ sub newsrc_put
 {
     local($fname, $ng, $ranges) = @_;
 
-    &getrc($fname) if ! $newsrc{0};
-    if (! defined($newsrc{$ng}))
+    &getrc($fname) unless $newsrc{0};
+    unless (defined($newsrc{$ng}))
     {
 	push(@ngorder, $ng);
 	$subscribed{$ng} = 1;
@@ -558,7 +591,7 @@ sub newsrc_merge
     local($ng, $ranges) = @_;
 
 #  print "merging <$ng> <$newsrc{$ng}> with ranges <$ranges>\n";
-    &getrc($fname) if !$newsrc{0};
+    &getrc($fname) unless $newsrc{0};
     if ($newNewsGroup{$ng})
     {
 #      print "merging new newsgroup <$ng>, inserting at pos $#ngorder and subscribe\n";
@@ -663,7 +696,7 @@ sub canon_inverse
 
     # find overlapping articles
     @l = grep(/(\d+)-(\d+)/ && ($1 <= $hi && $2 >= $lo), @l);
-    return ("$lo-$hi") if ! @l;
+    return ("$lo-$hi") unless @l;
 
 # print "overlapping <", join(',',@l), ">\n";
 
@@ -703,7 +736,7 @@ sub canon_isect
     {
 	($lo, $hi) = split(/-/, $el);
 	@overlap = grep(/(\d+)-(\d+)/ && ($1 <= $hi && $2 >= $lo), @second);
-	next if ! @overlap;	# save time if no overlapping ranges
+	next unless @overlap;	# save time if no overlapping ranges
 
 	# fix up the first and last overlapping ranges
         $overlap[0] = "$lo-$2"
